# -*- coding: utf-8 -*-
"""
    wakatime.language_priorities
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Overwrite pygments Lexer.priority attribute for specific languages.

    :copyright: (c) 2017 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


LANGUAGES = {
    'f#': 0.01,
    'perl': 0.01,
    'perl6': 0.01,
    'typescript': 0.01,
}
